#include <math.h>
#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "C_EvalY.c"
#include "C_fcoste.c"
#include "C_rankY.c"

/******************************************************************************/

#define T 15
#define N 200000
#define M 3
#define ypos_size 5768

/******************************************************************************/

float mu_col[] = { 800, 950, 200, 900, 800, 150, 650, 800, 900, 300, 150, 600, 500, 150, 700 };
float d[T * N];
float Sbase[T * M];
float beta = 0.95;
float CV = 0.25;
float cBatch = 1000, cUnit = 2, cWaste = 0, cSaveInv = 0.5;
float mincost = 1.0 / 0.0; //=inf
int bestindex;

/******************************************************************************/

int *FirstOrder_RoundRobin(int *input, int *numElements)
{
	int i;
	*numElements = 0;

	int *P = (int *)malloc(ypos_size * sizeof(int));
	if (P == NULL) {
		printf("An error has occured while creating P.\n");
		exit(1);
	}
	
	for (i = 0; i < ypos_size; i++) {
			P[*numElements] = i;
			*numElements = *numElements + 1;
	}

	return P;
}

/******************************************************************************/

int *SecondOrderByPrOrder(int *input, int *numElements)
{
	int i;
	*numElements = 0;

	int *P = (int *)malloc(ypos_size * sizeof(int));
	if (P == NULL) {
		printf("An error has occured while creating P.\n");
		exit(1);
	}

	for (i = 0; i < ypos_size; i++) {
			P[*numElements] = i;
			*numElements = *numElements + 1;
	}

	return P;
}

/******************************************************************************/

int *SecondOrderByPr(int *input, int *numElements)
{
	int i;
	*numElements = 0;

	int *P = (int *)malloc(ypos_size * sizeof(int));
	if (P == NULL) {
		printf("An error has occured while creating P.\n");
		exit(1);
	}

	for (i = 0; i < ypos_size; i++) {
			P[*numElements] = i;
			*numElements = *numElements + 1;
	}

	return P;
}

/******************************************************************************/

void mainloop(int numElements, int *ypos) {
	int i;
	int Y[T];
	float Qoptimal[T];
	float *I;
	float *X;

	I = (float *) malloc(T * N * M * sizeof(float));
	if (I == NULL) {
		printf("An error has occured while creating I.\n");
		exit(1);
	}
	X = (float *) malloc(T * N * sizeof(float));
	if (X == NULL) {
		printf("An error has occured while creating X.\n");
		exit(1);
	}

	for (i = 0; i < numElements; i++) {
		memcpy(Y, &ypos[i * T], T * sizeof(int));
		memset(Qoptimal, 0, T * sizeof(float));
		C_EvalY(Y, I, X, Sbase, T, M, N, d, mu_col, beta, CV, Qoptimal, bestindex, Qoptimal);

		float c = C_fcoste(Qoptimal, Y, I, X, T, M, N, cBatch, cSaveInv, cUnit, cWaste);
		if (c < mincost) {
			mincost = c;
			bestindex = i;
		}
	}
}

/******************************************************************************/

/*
 * Main program
 */
int main(int argc, char **argv) {
	FILE *fdemand;
	FILE *fypos;
	FILE *fsbase;
	int i = 0;
	int heur;
	heur = atoi(argv[1]);
	printf("heur:%d\n", heur);
	struct timeval timegettimeofday1, timegettimeofday2;
	gettimeofday(&timegettimeofday1, NULL);

	int *P;
	int numElements;

	int *ypos;
	ypos = (int *) malloc(T * ypos_size * sizeof(int));
	if (ypos == NULL) {
		printf("\n Error creating ypos");
	}

	fdemand = fopen("demanda200k15pnew.dat", "r");
	i = 0;
	while (fscanf(fdemand, "%f", &d[i]) == 1) {
		i = i + 1;
	}
	fclose(fdemand);

	i = 0;
	fsbase = fopen("Sbase15.dat", "r");
	while (fscanf(fsbase, "%f", &Sbase[i]) == 1) {
		i = i + 1;
	}
	fclose(fsbase);

	i = 0;
	int cont[T];
	memset(cont, 0, T * sizeof(int));

	int Y2[T];
	int valY[ypos_size];
	switch (heur) {
	case 0: //random
		printf("random order\n");
		fypos = fopen("yposrandom15p.txt", "r");
		while (fscanf(fypos, "%d", &ypos[i]) == 1) {
			i = i + 1;
		}
		fclose(fypos);
		for (i = 0; i < ypos_size; i++) {
			memcpy(Y2, &ypos[i * T], T * sizeof(int));
			valY[i] = C_rankY(Y2, T, M);
		}
		P = FirstOrder_RoundRobin(valY, &numElements); //Round-Robin without a sorted file is a blind distribution
		break;
	case 1: //Robin
		printf("Heur1 round robin\n");
		fypos = fopen("ypossort15p.txt", "r");
		while (fscanf(fypos, "%d", &ypos[i]) == 1) {
			i = i + 1;
		}
		fclose(fypos);
		for (i = 0; i < ypos_size; i++) {
			memcpy(Y2, &ypos[i * T], T * sizeof(int));
			valY[i] = C_rankY(Y2, T, M);
		}
		P = FirstOrder_RoundRobin(valY, &numElements);
		break;
	case 2: //H2
		printf("Heur2\n");
		fypos = fopen("yposrandom15p.txt", "r");
		while (fscanf(fypos, "%d", &ypos[i]) == 1) {
			i = i + 1;
		}
		fclose(fypos);
		for (i = 0; i < ypos_size; i++) {
			memcpy(Y2, &ypos[i * T], T * sizeof(int));
			valY[i] = C_rankY(Y2, T, M);
		}
		P = SecondOrderByPr(valY, &numElements);
		break;
	case 3: //H3
		printf("Heur3\n");
		fypos = fopen("ypossort15p.txt", "r");
		while (fscanf(fypos, "%d", &ypos[i]) == 1) {
			i = i + 1;
		}
		fclose(fypos);
		int Y2[T];
		int valY[ypos_size];
		for (i = 0; i < ypos_size; i++) {
			memcpy(Y2, &ypos[i * T], T * sizeof(int));
			valY[i] = C_rankY(Y2, T, M);
		}
		P = SecondOrderByPrOrder(valY, &numElements);
		break;
	default:
		printf("Unknown heuristic\n");
		exit(1);
	}

	printf("numElements = %d\n", numElements);

	mainloop(numElements, ypos);

	printf("Minimum cost: %f\n", mincost);

	gettimeofday(&timegettimeofday2, NULL);
	double elapsed = ((timegettimeofday2.tv_sec - timegettimeofday1.tv_sec) * 1000000LL +
			           timegettimeofday2.tv_usec - timegettimeofday1.tv_usec) / 1000000.0;
	printf("Elapsed time: %f sec\n", elapsed);

	return 0;
}

/******************************************************************************/
